package org.bitbucket.bradleysmithllc.etlunit.feature.informatica;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;
import org.bitbucket.bradleysmithllc.json.validator.JsonUtils;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/*
		default-domain: 'Domain_etldev01',
		domains: {
			Domain_etldev01: {
				client-version: '9.1.0hf3',
				connectivity-host: 'etldev01',
				connectivity-port: 6005,
				username: 'bsmith',
				password-encrypted: '',
				repositories: {
					dev_pc_repo: {
						integration-services: [
							'dev_ci_is',
							'dev_is',
							'dev_java_is'
						]
					}
				}
			}
			(
			,
			Domain_etlqa01: {
				client-version: '9.1.0hf3',
				connectivity-host: 'etlqa01',
				connectivity-port: 6005,
				username: 'bsmith',
				password-encrypted: '',
				repositories: {
					qa_pc_repo: {
						integration-services: [
							'qa_is'
						]
					}
				}
			}
			)*
		}
*/
public class InformaticaConfiguration {
	private final InformaticaDomain defaultDomain;
	private final Map<String, InformaticaDomain> domains = new HashMap<String, InformaticaDomain>();

	public InformaticaConfiguration(InformaticaFeatureModule mod, ETLTestValueObject value, RuntimeSupport runtimeSupport, File domainsFile)
	{
		this(mod, value.getJsonNode(), runtimeSupport, domainsFile);
	}

	public InformaticaConfiguration(InformaticaFeatureModule mod, JsonNode value, RuntimeSupport runtimeSupport, File domainsFile)
	{
		JsonNode edomains = JsonUtils.query(value, "domains");

		if (edomains == null)
		{
			throw new IllegalArgumentException("Missing domains");
		}

		if (!edomains.isObject())
		{
			throw new IllegalArgumentException("Domains element must be an object");
		}

		ObjectNode dlist = (ObjectNode) edomains;

		if (dlist.size() == 0)
		{
			throw new IllegalArgumentException("Domains element does not contain any entries");
		}

		Iterator<Map.Entry<String,JsonNode>> dit = dlist.fields();

		InformaticaDomain firstInformaticaDomain = null;

		while (dit.hasNext())
		{
			Map.Entry<String, JsonNode> dom = dit.next();

			InformaticaDomain informaticaDomain = new InformaticaDomain(mod, dom.getKey(), dom.getValue(), runtimeSupport, domainsFile);

			if (domains.containsKey(dom.getKey()))
			{
				throw new IllegalArgumentException("Domains element contains the same domain repeated: " + dom.getKey());
			}
			else
			{
				domains.put(dom.getKey(), informaticaDomain);
			}

			if (firstInformaticaDomain == null)
			{
				firstInformaticaDomain = informaticaDomain;
			}
		}

		JsonNode idefDomain = JsonUtils.query(value, "default-domain");

		if (idefDomain == null)
		{
			defaultDomain = firstInformaticaDomain;
		}
		else
		{
			String name = idefDomain.asText();

			defaultDomain = domains.get(name);

			if (defaultDomain == null)
			{
				throw new IllegalArgumentException("Default domain " + name + " not found in domains list");
			}
		}
	}

	public InformaticaDomain getDefaultDomain() {
		return defaultDomain;
	}

	public Map<String, InformaticaDomain> getDomains() {
		return domains;
	}

	public InformaticaDomain getDomain(String name) {
		InformaticaDomain dom = domains.get(name);

		if (dom == null)
		{
			throw new IllegalArgumentException("Domain not found " + name);
		}

		return dom;
	}
}